"use strict";

var debug = require("debug")("generator-isocket-cortex:app");
var path = require("path");
var npmName = require("npm-name");
var yeoman = require("yeoman-generator");
var yosay = require("yosay");

var iSocketCortexGenerator = yeoman.generators.Base.extend({
  initializing: function () {
    debug("initializing");
    this.pkg = require("../package.json");

    this.log(yosay("Welcome to the super iSocket Cortex generator!"));

    this.log(
      "\nThe module name should start with \"isocket-cortex-\" and" +
      "\nbe a unique ID not already in use at search.npmjs.org." +
      "\nIt shouldn\"t contain \"node\" or \"js\""
    );
  },

  prompting: {
    askForModuleName: function () {
      debug("asking for module name");
      var done = this.async();

      var prompts = [
        {
          name: "name",
          message: "Module Name",
          default: path.basename(process.cwd())
        }, {
          type: "confirm",
          name: "pkgName",
          message: "The name above already exists on npm, choose another?",
          default: true,
          when: function(answers) {
            var doneHere = this.async();

            npmName(answers.name, function (err, available) {
              debug("npm name available: " + available);
              doneHere(err || !available);
            });
          }
        }
      ];

      var processPrompts = function (props) {
        if (props.pkgName) {
          return this.askForModuleName();
        }

        this.slugname = this._.slugify(props.name);
        this.safeSlugname = this.slugname.replace(/-+([a-zA-Z0-9])/g,
          function (g) { return g[1].toUpperCase(); }
        );

        done();
      };

      this.prompt(prompts, processPrompts.bind(this));
    },

    askFor: function () {
      var cb = this.async();

      var prompts = [
        {
          name: "description",
          message: "Description"
        }, {
          name: "authorName",
          message: "Author\"s Name"
        }, {
          name: "authorEmail",
          message: "Author\"s Email"
        }, {
          name: "keywords",
          message: "Key your keywords (comma to split)"
        }, {
          name: "private",
          message: "Is this a private module?",
          default: "yes"
        }
      ];

      var processProps = function (props) {
        debug("got props from prompt: " + JSON.stringify(props));

        this.repoUrl = "https://github.com/isocket/" + this.slugname;
        debug("repoUrl: " + this.repoUrl);

        this.keywords = props.keywords.split(",");
        debug("keywords: " + JSON.stringify(this.keywords));

        this.props = props;
        debug("props: " + JSON.stringify(props));

        cb();
      };

      this.prompt(prompts, processProps.bind(this));
    }
  },

  configuring: {

    module: function () {
      debug("starting copying and templating module files");

      this.config.save();

      this.copy("editorconfig", ".editorconfig");
      this.copy("gitignore", ".gitignore");
      this.copy("eslintrc", ".eslintrc");
      this.copy("travis.yml", ".travis.yml");

      this.template("_README.md", "README.md");
      this.template("_package.json", "package.json");

      debug("finished copying and templating module files");
    },

    projectfiles: function () {
      debug("starting creating project files");

      this.mkdir("lib");
      this.template("lib/name.js", "lib/" + this.slugname + ".js");

      this.mkdir("test");
      this.template("test/name.js", "test/" + this.slugname + ".js");

      this.mkdir("example");
      this.template("example/name.js", "example/" + this.slugname + ".js");

      debug("finished creating project files");
    }
  },

  install: function () {
    debug("started installing");
    debug("skipping install? " + this.options["skip-install"]);
    this.installDependencies({
      bower: false,
      skipInstall: this.options["skip-install"] // important for testing
    });

    this.log(yosay("Remember to run `travis setup npm` if you\"re deploying to npm"));
    debug("finished installing");
  }
});

module.exports = iSocketCortexGenerator;
